%CALCULATIONS on RC AND PCB RC
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%CONSTANT PARAMETERS
k1 = 1*10^3;
k2 = 1*10^6;
k3 = 1*10^-3;
Irms = 100;                                         %Current RMS
I0 = Irms*sqrt(2);                                  %Current amplitude
f = 50;                                             %Frequency
w = 2*pi*f;                                         %Omega
mu0 = 4*pi*10^-7;                                   %Permittivity of free space

% PCB RC
router = 46*10^-3;                                  %Outer radius
rinner = 22 *10^-3;                                 %Inner radius
rcenter = rinner + ((router-rinner)/2);             %To the coil center
d = 2*10^-3;                                        %thickness
L = 2*pi*rcenter;                                   %Circumference
N =62;                                              %# turns
nn = N/L;                                           %# turns per meter
t = [0:0.5*10^-3:50*10^-3];                         %Time, 2.5 power cycles
I = I0*sin(w*t);                                    %Primary conductor current
Nvar = [0:100:1000];                                %Varying N from 0 -> 1000 turns

%RC
layers = 6;                                         %Must be even number if > 1
wireThickness = 0.8*10^-3;                          %Determines # turns
rRCinner = 50*10^-3 - (wireThickness*layers);       %Only add last term if layers > 1
rRCouter = 110*10^-3 + (wireThickness*layers);      %Only add last term if layers > 1
rRCcenter = rRCinner + ((rRCouter-rRCinner)/2);
rCrossSection = (rRCouter-rRCinner)/2;
rCore = rCrossSection - (wireThickness*layers);
RA = rRCinner + (wireThickness*layers);
Linner = 2*pi*rRCinner;
Lcenter = 2*pi*rRCcenter;
NRC = layers*(Linner/wireThickness);
nRCcenter = NRC/Lcenter;

%CURRENT DERIVATIVE
dI = I0*w*cos(w*t);


%SIMPLE MODEL e = mu0*n*A*dI FOR PCB RC + RC
Areal = (router-rinner)*d;
Asirk = pi*(rCrossSection^2);
Msimple = mu0*nn*Areal;
MRC = mu0*nRCcenter*Asirk;
e1 = Msimple*dI;
eRC = MRC*dI;
RCsensitivity = max(eRC*k1) / I0;

%DETAILED MODEL e = (mu0*N*d*ln(router/rinner)*dI)/2*pi for RC
Mdetailed = mu0*N*d*log(router/rinner)/(2*pi);
e2 = Mdetailed*dI;

%LAYER IMPACT RC
lay = [1,2:2:6];
d_wire = 1*10^-3*[0.3, 0.4, 0.8];
for n = 1:3
    for m = 1:length(lay)
        if (lay(m) == 1)
            r_lay_inner = 50*10^-3;
            r_lay_outer = 80*10^-3;
        else
            r_lay_inner = 50*10^-3 - (d_wire(n)*lay(m));
            r_lay_outer = 80*10^-3 + (d_wire(n)*lay(m));
        end
        r_lay_cross = (r_lay_outer-r_lay_inner)/2;
        r_lay_center = r_lay_inner + r_lay_cross;
        L_lay_inner = 2*pi*r_lay_inner;
        L_lay_center = 2*pi*r_lay_center;
        A_lay = pi*(r_lay_cross^2);
        N_lay = lay(m)*(L_lay_inner/d_wire(n));
        n_lay = N_lay/L_lay_center;
        M_lay = mu0*n_lay*A_lay;
        e_lay = M_lay*dI;
        e_lay_maks(m) = max(e_lay);
    end
    figure(15)
    hold on
    plot(lay,e_lay_maks)
    axis([1 6 0 1])
    xlabel('Number of layers')
    ylabel('Output voltage [V]')
    box on
    grid on
end
hold off

%CORE LENGTH IMPACT
ra = 1*10^-3*[50:5:300];
d_wire = 0.3*10^-3;
lag = 6;
r_core = 15*10^-3;
r_core_w = r_core + (lag*d_wire);
ra_w = ra - (lag*d_wire);
rb = ra + (2*r_core);
rb_w = rb + (lag*d_wire);

for i = 1:length(ra)
    r_center(i) = ra_w(i) + r_core + (lag*d_wire);
    v_corelength(i) = k1*mu0*lag*ra_w(i)*pi*(r_core_w^2)*max(dI)/(d_wire*r_center(i));%mV
end

%CORE RADIUS IMPACT
Ra = 50*10^-3;
Ra_w = Ra - (lag*d_wire);
R_core = 1*10^-3*[15:1:30];
R_core_w = R_core + (lag*d_wire);

for i = 1:length(R_core)
    R_center(i) = Ra + R_core(i);
    v_coreradius(i) = mu0*lag*Ra_w*pi*(R_core_w(i)^2)*max(dI)/(d_wire*R_center(i));%mV
end

%TURN IMPACT
figure(11) %induced voltage/time
plot(t*k1, e2*k1,'k')
hold on
for i = 1:length(Nvar)
    Mvar(i) = mu0*Nvar(i)*d*log(router/rinner)/(2*pi);
    evar = Mvar(i)*dI;
    plot(t*k1,evar*k1)
end
xlabel('Time [ms]')
ylabel('Induced voltage [mV]')
hold off

%NUMBER OF TURNS TO GET X VOLT AMPLITUDE PCB RC
xmaks = 1;  %Volt
Nneed = (xmaks*2*pi)/(mu0*d*dI(1)*log(router/rinner));  %Calculates how many N is needed
Voltvar = 0:10*10^-3:1;
for i = 1:length(Voltvar)
    Nneeded(i) = (Voltvar(i)*2*pi)/(mu0*d*dI(1)*log(router/rinner));
end
figure(12)%induced voltage / #turn
plot(Nneeded*10^-3, Voltvar)
xlabel('Number of turns, in thousands')
ylabel('Induced voltage amplitude [V]')
axis([0 50 0 0.71])
grid on

%VOLTAGE TO CURRENT RATIO
figure(13) %integrated voltage signal and current to be measured / time
eintegrert = Mdetailed*w*I;
ratio = eintegrert./(I);
sensitivity = sum(ratio)/length(ratio);
plot(t*k1,eintegrert*k1, t*k1, I*10^-3)
legend('Integrated voltage signal [V]', 'Current to be measured [A]')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%PLOT AND OUTPUT
figure(1)
plot(t*k1,I,t*10^3,dI*10^-3)
legend('Primary current [A]', 'Location', 'NorthOutside',...
    'Current derivative [kA/s]', 'Location', 'NorthOutside');
ylabel('Current')
xlabel('Time [ms]')
grid on;

figure(14) %induced voltage rc / time
plot(t*k1, eRC*k1, [0 50],[0 0], 'k')
legend('RC Simple model [mV]', 'Location', 'NorthOutside');
ylabel('Induced voltage')
xlabel('Time [ms]')

figure(16)
plot(ra*k1, v_corelength)
grid on
xlabel('Inner radius [mm]')
ylabel('Output voltage [mV]')

figure(17)
plot(k1*R_core, v_coreradius)
grid on
xlabel('Core radius [mm]')
ylabel('Output voltage [V]')

hold off
figure(2) %Output voltage signal PCB RC / time
plot(t*10^3, e1*10^3, t*10^3, e2*10^3,t*10^3, e1*10^3*1.8, t*10^3, e2*10^3*1.8,...
    'k', [0 50],[0 0], 'k')
legend('100 A, Equation (2.13) ', 'Location', ...
    'NorthOutside', '100 A, Equation (2.19) ', ...
    'Location', 'NorthOutside', '180 A, Equation (2.13) ', ...
    'Location', 'NorthOutside', '180 A, Equation (2.19) ');
ylabel('Output voltage signal [mV]')
xlabel('Time [ms]')

figure(10) %Mutual inductance / #turn
plot(Nvar,Mvar*k2*k1, [62 62], [0 18.29], 'k', [0 62], [18.29 18.29], 'k')
ylabel('Mutual inductance [nH]')
xlabel('Number of turns')
axis([0 1000 0 0.3*k1])
grid on
dMdN = Mvar./Nvar;

STR1 = ['Msimple = ', num2str(Msimple*k1*k2, 3), 'nH &&& Mdetailed = ',...
    num2str(Mdetailed*k1*k2, 3), 'nH &&&'];
STR2 = ['Circumference = ', num2str(L,3), 'm &&&',...
    ' ratio = ', num2str(ratio(2)), ' V/A'];
disp(STR1)
disp(STR2)
disp(layers)
disp(nRCcenter)
disp(NRC/layers)
disp(max(eRC*k1))   %peak in mV
disp(RCsensitivity) %mV / A
